@@ -68,7 +68,7 @@ class Gateway:
 
     async def on_connection(self, connection: websockets.WebSocketServerProtocol, path: str):
         self.logger.info(f'New connection from {connection.remote_address} path: {path}')
-        websocket_connection = WebsocketConnection(connection, request_handler=self.handle_request)
+        websocket_connection = WebsocketConnection(connection, request_handler=self.handle_request, close_handler=self.handle_close)
         self.websocket_connections.append(websocket_connection)
 
         await websocket_connection.start()
@@ -121,6 +121,18 @@ class Gateway:
 
         return messages.Response(status=messages.Status.ERROR, data=errors.InvalidMethodError(f'Invalid method: {request.method}'))
 
+    async def handle_close(self, websocket_connection: WebsocketConnection):
+        if websocket_connection in self.websocket_instances:
+            # remove registered instances
+            for instance_name in self.websocket_instances[websocket_connection]:
+                self.instances.pop(instance_name)
+
+            self.websocket_instances.pop(websocket_connection)
+
+        # send an error response for any in-progress requests
+        for request_id, response_queue in websocket_connection.receive_queues.items():
+            await response_queue.put(messages.Response(status=messages.Status.ERROR, data=errors.ServerConnectionLostError(f'Server connection lost')))
+
 
 class GatewayClient:
     def __init__(self, gateway_url: str = 'ws://localhost:8888'):
