@@ -3,6 +3,7 @@ from inspect import isclass
 from os import path
 import json
 import yaml
+from deepmerge import always_merger
 
 FactoryType = Callable[..., Any]
 ConfigurationType = Dict[str, Union[List, Dict[str, Any]]]
@@ -28,6 +29,7 @@ def default_registry() -> 'Registry':
 class Registry:
     CONFIG_META_PREFIX = '$'
     ARGS_KEY = '$args'
+    TYPE_KEY = '$type'
     _default_registry: Optional['Registry'] = None
 
     @staticmethod
@@ -64,24 +66,28 @@ class Registry:
 
     def configure(self, config: Union[str, ConfigurationType], file_type: Optional[str] = None):
         if isinstance(config, dict):
-            self.configuration = config
+            self.merge_configuration(config)
         else:
             self.load_configuration(config, file_type)
 
+    def merge_configuration(self, config: ConfigurationType):
+        self.configuration = always_merger.merge(self.configuration or {}, config)
+
     def load_configuration(self, config_path: Optional[str] = None, file_type: Optional[str] = None):
         file_path = config_path or self.config_path
 
+        # use the file extension as the file type if none is given
         if file_type is None:
             _, config_ext = path.splitext(file_path)
-            config_type = config_ext[1:]
+            config_type = config_ext[1:]  # remove .
         else:
             config_type = file_type
 
         with open(file_path or self.config_path) as config_file:
             if config_type == 'json':
-                self.configuration = json.load(config_file)
+                self.merge_configuration(json.load(config_file))
             elif config_type == 'yaml':
-                self.configuration = yaml.safe_load(config_file)
+                self.merge_configuration(yaml.safe_load(config_file))
             else:
                 raise RegistryError(f'Invalid config file format, must be "json" or "yaml": {config_type}')
 
@@ -93,8 +99,8 @@ class Registry:
         # remove any keys that don't start with @ (and aren't @args) and remove the @ from the keys
         return {k[1:]: v for k, v in config.items() if k.startswith(self.CONFIG_META_PREFIX) and k != self.ARGS_KEY}
 
-    def get_object_configuration(self, object_name: str, *args: Any, **kwargs: Any) -> Tuple[List, Dict]:
-        if object_name not in self.configuration:
+    def get_object_configuration(self, object_name: Optional[str], *args: Any, **kwargs: Any) -> Tuple[List, Dict]:
+        if object_name is None or object_name not in self.configuration:
             return [], {}
 
         config = self.configuration[object_name]
@@ -102,10 +108,35 @@ class Registry:
         if isinstance(config, list):
             return [*config, *args], kwargs
 
-        config_args = config.get(self.ARGS_KEY, [])
-        config_kwargs = {k: v for k, v in config.items() if not k.startswith(self.CONFIG_META_PREFIX)}
+        # start with the config of the factory defined by the $type key in the config, if it exists
+        config_args, config_kwargs = self.get_object_configuration(config.get(self.TYPE_KEY, None))
+
+        # merge object config, if defined
+        config_args = [*config_args, *config.get(self.ARGS_KEY, [])]
+        config_kwargs = {**config_kwargs, **{k: v for k, v in config.items() if not k.startswith(self.CONFIG_META_PREFIX)}}
+
+        # finally merge in any additional args and kwargs
         return [*config_args, *args], {**config_kwargs, **kwargs}
 
+    def get_object_factory(self, object_name: str) -> FactoryType:
+        if object_name not in self.factories:
+            if object_name not in self.configuration:
+                raise RegistryFactoryNotFoundError(f'Unable to find a factory or config for object: {object_name}')
+
+            if self.TYPE_KEY not in self.configuration[object_name]:
+                raise RegistryFactoryNotFoundError(
+                    f'No factory or "$type" key found in config for object: {object_name}')
+
+            factory_name = self.configuration[object_name][self.TYPE_KEY]
+
+            if factory_name not in self.configuration:
+                raise RegistryFactoryNotFoundError(
+                    f'Unable to find a factory for "{object_name}" with $type: {factory_name}')
+
+            return self.factories[factory_name]
+
+        return self.factories[object_name]
+
     def register(self, factory: FactoryType, name: Optional[str] = None):
         if name is None:
             if not isclass(factory):
@@ -120,12 +151,9 @@ class Registry:
         self.factory_types[factory] = self.factory_types.get(factory, []) + [name]
 
     def create_object(self, object_name: str, *args: Any, **kwargs: Any):
-        if object_name not in self.factories:
-            raise RegistryFactoryNotFoundError(f'Unable to find a factory for object: {object_name}')
-
+        factory = self.get_object_factory(object_name)
         object_args, object_kwargs = self.get_object_configuration(object_name, *args, **kwargs)
-
-        return self.factories[object_name](*object_args, **object_kwargs)
+        return factory(*object_args, **object_kwargs)
 
     def get_by_name(self, object_name: str, *args: Any, **kwargs: Any):
         if object_name in self.objects:
